<script src="https://cdnjs.cloudflare.com/ajax/libs/spin.js/2.3.2/spin.min.js"></script>
<script crossorigin src="https://unpkg.com/react@16/umd/react.production.min.js"></script>
<script crossorigin src="https://unpkg.com/react-dom@16/umd/react-dom.production.min.js"></script>
<script crossorigin src="https://unpkg.com/three@0.126.1/build/three.min.js"></script>
<script crossorigin src="https://unpkg.com/@acdh/network-visualization@0/lib/network-visualization.umd.js"></script>